<?php
/**
 * @copyright 2018 Shift Tech Inc All Rights Reserved.
 * @author Shift Tech inc.<https://www.shift-tech.jp/>
 */

if(!defined('URL_GMAP_API')) {
    define('URL_GMAP_API', 'https://maps.googleapis.com/maps/api/js');
}


require_once (LIB_PATH.'/common/unitCommon.php');
include_once (LIB_PATH.'/common/imgmngrDB.php');

class extMap extends unitCommon
{
    //------ ユニット毎に必ず設定しなくてはならない変数 -------//
    var $_table = "extunit_map";        //公開用テーブル名
    var $_tmptable = "extunit_map_tmp";    //下書き用テーブル名
    var $columns = array(
        array("column"=>"pageu_code","conv"=>"n","tags"=>false),
        array("column"=>"bmap_service","conv"=>"n","tags"=>false),
        array("column"=>"bmap_apikey","conv"=>"","tags"=>false),
        array("column"=>"bmap_gapikey","conv"=>"","tags"=>false),
        array("column"=>"bmap_img","conv"=>"","tags"=>false),
        array("column"=>"bmap_title","conv"=>"KVa","tags"=>false),
        array("column"=>"bmap_addr","conv"=>"KVa","tags"=>false),
        array("column"=>"bmap_body","conv"=>"KVa","tags"=>false),
        array("column"=>"bmap_lat","conv"=>"","tags"=>false),
        array("column"=>"bmap_lng","conv"=>"","tags"=>false),
        array("column"=>"bmap_height","conv"=>"n","tags"=>false),
        array("column"=>"bmap_height_sp","conv"=>"n","tags"=>false),
        array("column"=>"use_sp_flg","conv"=>"n","tags"=>false),
        array("column"=>"use_pc_flg","conv"=>"n","tags"=>false),
        array("column"=>"bg_color","conv"=>"n","tags"=>false),        //背景色
        array("column"=>"bdr_type","conv"=>"n","tags"=>false),        //ボーダー
        array("column"=>"btm_padding","conv"=>"n","tags"=>false),      //下方スペース
        array("column"=>"bg_color_sp","conv"=>"n","tags"=>false),     //背景色
        array("column"=>"bdr_type_sp","conv"=>"n","tags"=>false),    //ボーダー
        array("column"=>"btm_padding_sp","conv"=>"n","tags"=>false),   //下方スペース
    );

    var $_service = array(
        0 => 'Yahoo!地図',
        1 => 'Google マップ',
    );

    function extLine()
    {
        $this->__construct();
    }

    function __construct()
    {
        unitCommon::__construct();
    }

    //出力データセット処理（必須）
    function setDisplayData() {
        $this->_data = $this->getDetail();

        if ($this->_data) {
            if (defined('SP_FLG') && SP_FLG == SP_FLG_CODE) {
                if (isset($this->_data['bmap_height_sp']) && is_numeric($this->_data['bmap_height_sp'])) {
                    $this->_data['bmap_height'] = $this->_data['bmap_height_sp'];
                }
            }
            if ($this->_prev) {
                $this->_data['prev'] = $this->_prev;
            }
        }

        return $this->_data;
    }

    /*
     * DBより保存データ取得
     * 改変可能
     */
    function getDetail() {
        $rtn =  null;
        $sql = "select * from ".$this->getTbl()." where pageu_code = ?";
        if(isset($this->_pageu_code)&&is_numeric($this->_pageu_code)) {
            $result = $this->db->prepare($sql,$this->_pageu_code);
            if($result){
                $rtn = $result->fetchRow(DB_FETCHMODE_ASSOC);
                $gapikey = $rtn['bmap_gapikey'];
                if($_POST['bmap_gapikey']) {
                    $gapikey = $_POST['bmap_gapikey'];
                }
                if(!defined('GOOGLE_API_KEY')) {
                    define('GOOGLE_API_KEY', $gapikey);
                }
            }
        }
        return $rtn;
    }


    function setDefault()
    {

        $data = null;
        $data = $this->getDetail();
        $service = 0;
        if ($data) {
            $this->_form->setDefaults($data);

            if(isset($_POST['bmap_service'])) {
                $service = $_POST['bmap_service'];
            } else {
                $service = $data['bmap_service'];
            }
        } else {
            $data['btm_padding_sp'] = 2;
            $data['bmap_height'] = 300;
            $data['bmap_apikey'] = "";
            $data['bmap_gapikey'] = "";
            if(isset($_POST['bmap_service'])) {
                $service = $_POST['bmap_service'];
            }
            //api id の取得
            $apikey = null;
            $sql = 'select bmap_apikey from '.$this->getTbl().' order by pageu_code desc limit 1';
            $result = $this->db->getOne($sql);
            if ($result) {
                $apikey = $result;
                $data['bmap_apikey'] = $apikey;
            }

            $sql = 'select bmap_gapikey from '.$this->getTbl().' order by pageu_code desc limit 1';
            $result = $this->db->getOne($sql);
            if ($result) {
                $apikey = $result;
                $data['bmap_gapikey'] = $apikey;
            }

            $this->_form->setDefaults($data);
        }

        // if(empty($site['sbase_google_api_key'])) {
        //     $service = 0;
        // }

        $this->_view->assign('service', $service);
    }

    function getSite()
    {
        $res = $this->db->query('SELECT * FROM tbl_sitebase ORDER BY sbase_code ASC LIMIT 1 OFFSET 0');
        if($res) {
            return $res->fetchRow(DB_FETCHMODE_ASSOC);
        }

        return array();
    }

    /**
     * フォームエレメントの作成.
     */
    public function setElements()
    {
        if(class_exists('bConfig')) {
            $site = bConfig::get("SITEINFO");
        } else {
            $site = $this->getSite();

        }
        $this->_form->addElement('hidden', 'pageu_code', $this->_pageu_code, array('id' => 'pageu_code'));
        $this->_form->addElement('hidden', 'bmap_img', null, array('id' => 'bmap_img'));
        $this->_form->addElement('file', 'picture', '');


        // if(empty($site['sbase_google_api_key'])) {
        //     // $this->_form->addElement('hidden', 'bmap_service', 0, array());
        //     $this->_form->addElement('select', 'bmap_service', null, array(0 => 'Yahoo!地図'), array('id' => 'bmap_service'));
        // } else {
        //     $this->_form->addElement('select', 'bmap_service', null, $this->_service, array('id' => 'bmap_service'));
        // }
        $this->_form->addElement('select', 'bmap_service', null, $this->_service, array('id' => 'bmap_service'));

        $this->_form->addElement('text', 'bmap_title', null, array('id' => 'bmap_title', 'size' => 40, 'maxlength' => 40));
        $this->_form->addElement('text', 'bmap_addr', null, array('id' => 'bmap_addr', 'size' => 40, 'maxlength' => 40));
        $this->_form->addElement('textarea', 'bmap_body', null, array('id' => 'bmap_body', 'rows' => 5));

        $this->_form->addElement('text', 'bmap_apikey', null, array('id' => 'bmap_apikey', 'size' => 40, 'maxlength' => 200));
        $this->_form->addElement('text', 'bmap_gapikey', null, array('id' => 'bmap_gapikey', 'size' => 40, 'maxlength' => 200));

        $this->_form->addElement('text', 'bmap_height', null, array('id' => 'bmap_height', 'size' => 20, 'maxlength' => 20, 'style' => 'ime-mode:disabled'));
        $this->_form->addElement('text', 'bmap_height_sp', null, array('id' => 'bmap_height_sp', 'size' => 20, 'maxlength' => 20, 'style' => 'ime-mode:disabled'));

        $this->_form->addElement('text', 'bmap_lat', null, array('id' => 'bmap_lat', 'size' => 20, 'maxlength' => 20, 'style' => 'ime-mode:disabled'));
        $this->_form->addElement('text', 'bmap_lng', null, array('id' => 'bmap_lng', 'size' => 20, 'maxlength' => 20, 'style' => 'ime-mode:disabled'));

        $this->_form->addElement('checkbox', 'pic_delflg', null, _gettext('B_P00400'), null);//画像削除チェックボックス
        $this->_form->updateElementAttr('pic_delflg', array('value' => 'delete'));


        $this->_form->addElement('checkbox', 'bg_color_sp', null, _gettext('B_P00530'), null);
        $this->_form->addElement('select', 'bdr_type_sp', null, $this->_unit_decoration['bdr_type_set']);
        $this->_form->addElement('select', 'btm_padding_sp', null, $this->_unit_decoration['btm_padding_set']);
    }
    /**
     * フォーム入力エラーチェックの設定.
     */
    public function setRules()
    {
        $this->_form->applyFilter('__ALL__', 'trim');
        $this->_form->_submitValues = DataBase::ConvColumnVals($this->_form->_submitValues, $this->columns); //ルールに従った値変換、タグの除去
        $this->_form->setJsWarnings(_gettext('B_P00472'), '');
        $this->_form->addRule('pageu_code', _gettext('B_P00643'), 'required', null, $this->erchk);
        $obj = $this->_form->getElement('bmap_img');
        if ($obj->getValue() && file_exists(BASE_PATH.UNITS_IMG_PATH.$obj->getValue())) {
            $this->_view->assign('image', UNITS_IMG_PATH.$obj->getValue());
        }
        $this->_form->addRule('picture', _gettext('B_P00635', MAX_FILE_SIZE / (1000 * 1000)), 'maxfilesize', MAX_FILE_SIZE, null, 'server');
        $this->_form->addRule('picture', _gettext('B_P00171'), 'mimetype', array('image/pjpeg', 'image/jpeg', 'image/gif', 'image/x-png', 'image/png'));

        $this->_form->addRule('bmap_title', _gettext('B_P00753'), 'required', null, $this->erchk);
        $this->_form->addRule('bmap_addr', _gettext('B_P00754'), 'required', null, $this->erchk);
        if($_POST['bmap_service'] == 0) {
            $this->_form->addRule('bmap_apikey', _gettext('B_P00755'), 'required', null, $this->erchk);
        }
        if($_POST['bmap_service'] == 1) {
            $this->_form->addRule('bmap_gapikey', _gettext('・「Google Maps Platform APIキー」を入力してください'), 'required', null, $this->erchk);
        }
        $this->_form->addRule('bmap_height', _gettext('B_P00745'), 'required', null, $this->erchk);
        $this->_form->addRule('bmap_height', _gettext('B_P00746'), 'numeric', null, $this->erchk);
        $this->_form->addRule('bmap_height_sp', _gettext('B_P00746'), 'numeric', null, $this->erchk);
        $this->_form->registerRule('NumericRange_valid', 'callback', 'NumericRange');
        $this->_form->addRule('bmap_height', _gettext('B_P00747'), 'NumericRange_valid', array(50, 2000), $this->erchk);
        $this->_form->addRule('bmap_height_sp', _gettext('B_P00747'), 'NumericRange_valid', array(50, 2000), $this->erchk);
        $this->_form->addRule('bmap_lat', _gettext('B_P00756'), 'required', null, $this->erchk);
        $this->_form->addRule('bmap_lat', _gettext('B_P00757'), 'regex', '/[0-9-\.]+$/', $this->erchk);
        $this->_form->addRule('bmap_lng', _gettext('B_P00758'), 'required', null, $this->erchk);
        $this->_form->addRule('bmap_lng', _gettext('B_P00759'), 'regex', '/[0-9-\.]+$/', $this->erchk);
        $this->_form->registerRule('check_tags', 'callback', 'check_tags');
        $this->_form->addRule('bmap_body', _gettext('B_P00681'), 'check_tags', null, $this->erchk);
    }
    /**
     * データ下書き保存処理.
     *
     * @param array $post 保存データ
     *
     * @return null|string エラー時=string
     */
    public function doSave($post)
    {
        //あらかじめ画像処理
        $result = null;
        $bimg = '';
        //既存データがあるか確認（画像削除用にも使用）
        $old_data = $this->getDetail();

        $record = null;
        foreach ($this->columns as $val) {
            if ($post[$val['column']] != null) {
                $record[$val['column']] = $post[$val['column']];
            } else {
                $record[$val['column']] = null;
            }
        }
        // ここでDBに登録する画像情報を公開用の画像情報に設定する
        // 画像を差し替えた場合（たとえ同じ画像でも）、bimg_img には数値が入ってくる為、これを判定条件とする
        if (is_numeric($record['bmap_img'])) {
            // ここでhidden属性（上記の数値に設定されたもの）を現在設定されている画像パスの状態にもどす
            // 前提：エラー時（画像以外の入力でエラーになった場合でも）は新しく選択したファイルが画面上から消えること（既存ファイルが選択されている状態に戻る）
            if (is_array($old_data) && count($old_data) > 0) {
                $this->_form->updateElementAttr('bmap_img', array('value' => $old_data['bmap_img']));
            }
            $imgmngrDB = new imgmngrDB();
            $imgdata = $imgmngrDB->getImageFromCode($record['bmap_img'], UNITS_IMG_PATH, $record['pageu_code'], 1);
            if (isset($imgdata)) {
                $data_path = $imgdata['img_src']; // コピー元（素材管理の画像パス）
                //$img_path = UNITS_IMG_PATH . $record["pageu_code"] . '-1-' . date('YmdHis') . '.' . array_pop(explode('.',$data_path));  // コピー先
                $img_path = $imgdata['gimg_src'];  // コピー先（公開用の画像パス）
                $record['bmap_img'] = array_pop(explode('/', $img_path)); //保存ファイル名取得
                $cpflg = true;
            } else {
                return _gettext('B_P00651');
            }
        } elseif ($post['pic_delflg'] != null) {
            $record['bmap_img'] = null;
        }

        $this->db->query('BEGIN');
        //保存処理開始
        if (is_array($old_data) && count($old_data) > 0) {
            //更新
            if (!$this->db->update($this->getTbl(), $record, 'pageu_code='.$record['pageu_code'])) {
                $this->db->query('ROLLBACK');

                return _gettext('B_P00209');
            }
        } else { //新規
            if (!$this->db->insert($this->getTbl(), $record)) {
                $this->db->query('ROLLBACK');

                return _gettext('B_P00209');
            }
        }

        // saas
        $fsize = array();
        if ($cpflg) {
            $fsize[] = array('size' => filesize(BASE_PATH.$data_path));
        }
        if (count($fsize) > 0 && !commonFile::chkSaveFiles($fsize)) {
            $this->db->query('ROLLBACK');

            return _gettext('B_P00281');
        }

        // ここでファイルのコピーを行う
        if ($cpflg) {
            if (!@copy(BASE_PATH.$data_path, BASE_PATH.$img_path)) {
                $this->db->query('ROLLBACK');

                return _gettext('B_P00650');
            } else {
                @chmod(BASE_PATH.$img_path, 0777);
            }
        }
        $this->db->query('COMMIT');

        return;
    }
}
